# Copyright (c) YugaByte, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License.  You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.  See the License for the specific language governing permissions and limitations
# under the License.
#

import os

from .base import TestCase

from yugabyte_pycommon import mkdir_p
from random import randint


class FileSystemUtilTestCase(TestCase):
    def test_mkdir_p(self):
        tmp_dir_path = '/tmp/yugabyte_mkdir_p_test_%d' % randint(1, 1000000000)
        for _ in range(2):
            mkdir_p(tmp_dir_path)
            self.assertTrue(os.path.isdir(tmp_dir_path))
        os.rmdir(tmp_dir_path)